unit dmChart;

interface

uses
  System.Types, System.SysUtils, System.Classes, Vcl.Graphics, VclTee.TeeGDIPlus, VCLTee.TeEngine,
  VCLTee.TeeProcs, VCLTee.Chart, VCLTee.Series;

type
  TdmChartImage = class(TDataModule)
  private
    { Private declarations }
    procedure ChartAxisLabel(Sender: TChartAxis; Series: TChartSeries;
      ValueIndex: Integer; var LabelText: String);
    procedure ChartAfterDraw(Sender: TObject);
    procedure SetLegendPosition(Sender: TObject);
    procedure ChartBeforeDrawSeries(Sender: TObject);
  public
    { Public declarations }
    procedure CreateImage(aBmp: TBitmap);
  end;


implementation

uses
  math;


{$R *.dfm}

{ TDataModule2 }

procedure TdmChartImage.ChartAfterDraw(Sender: TObject);
begin
  SetLegendPosition(Sender);
end;

procedure TdmChartImage.ChartAxisLabel(Sender: TChartAxis; Series: TChartSeries;
  ValueIndex: Integer; var LabelText: String);
var
  lPos: Integer;
begin
  if (Sender.ParentChart.LeftAxis = Sender) and (Sender.Maximum >= 10000) then
  begin
    lPos := Pos(',', LabelText);
    if lPos > 0 then
      LabelText := Copy(LabelText, 1, lPos-1) + 'K';
  end;
end;

procedure TdmChartImage.ChartBeforeDrawSeries(Sender: TObject);
begin

end;

procedure TdmChartImage.CreateImage(aBmp: TBitmap);
var
  lChart: TChart;

  function CreateChartSeries(const aTitle: string): TAreaSeries;
  begin
    result := TAreaSeries.Create(self);
    result.ParentChart := lChart;
    result.AreaLinesPen.Visible := false;
    result.Pen.Visible := false;

    if lChart.SeriesCount > 1 then
      lChart.SeriesList.Move(lChart.SeriesCount - 1, 0);{}

    result.Title := aTitle;
  end;

const
  SCALE_FACTOR = 1;
var
  lRect: TRect;
  lsrs: TChartSeries;
  I: Integer;
  lBmp: TBitmap;
  //lValue: Double;
begin
  lChart.FreeAllSeries;

  for I := 1 to 4 do
  begin
    lsrs := CreateChartSeries('Series #' + IntToSTr(I));
    lsrs.FillSampleValues();
{    lValue := 1000;
    for J := 0 to 20 do
    begin
      lsrs.Add(lValue);
      lValue := lValue * 1.03;
    end;{}
  end;

  lChart.LeftAxis.Title.Caption := 'Anual Income';
  lChart.LeftAxis.Title.Font.Size := 10;
  lChart.BottomAxis.Title.Caption := 'Years in Retirement';{}
  lChart.View3D := false;
  lChart.BottomAxis.Title.Font.Size := 10;
  lChart.Legend.CustomPosition := True;
  lChart.Legend.Top := 05;
  lChart.Legend.Left := 05;

  lChart.OnBeforeDrawSeries := ChartBeforeDrawSeries;
  lChart.OnGetAxisLabel := ChartAxisLabel;
  lChart.OnAfterDraw := ChartAfterDraw;

  lRect := Rect(0, 0, aBmp.Width, aBmp.Height);
  lRect.Right := Ceil(lRect.Right * SCALE_FACTOR);
  lRect.Bottom := Ceil(lRect.Bottom * SCALE_FACTOR);

  lChart.Draw();
  lChart.Draw(); {}

  lBmp := lChart.TeeCreateBitmap(clWhite, lRect);
  try
    aBmp.Assign(lBmp);
  finally
    lBmp.Free;
  end;
end;

procedure TdmChartImage.SetLegendPosition(Sender: TObject);
var
  lChart: TChart;
  lLeft: Integer;
begin
  //reposition legend
  Assert(sender is TChart);
  lChart := TChart(Sender);
  if (lChart.SeriesCount > 0) and (lChart.Series[0].Count > 0) then
  begin
    lLeft := lChart.Series[0].CalcXPos(1);
    lChart.Legend.Left := lLeft;
  end;
end;

end.
